# Database URI when using Flask-SQLAlchemy, example: mysql+pymysql://username:password@server/db?charset=utf8mb4
# SQLALCHEMY_DATABASE_URI = ''


# guniflask configuration
guniflask = dict(
    cors=True,
)
